/**
 * Class DataBaseCordova : Gestion de la base sous Cordova (Application Mobile Android & Apple)
 */

var DatabaseCordovaMigrationService = (function () {
	
	var log = function (msg) {
		console.log('[SQL DATABASE] '+msg);
	};

	var executeSQL = function(db, query, args) {
		return new Promise(function (resolve, reject) {
			db.transaction(
				function (tx) {
					tx.executeSql(query, args, function(tx, results) {
				    	resolve(results, tx);
					});
				}, 
				function (err, sql) {
					reject(err, sql);
				},
				function () {
					resolve();
				}
			);
		});
	}

	var whenPromise = function(value) {
		return new Promise(function (resolve, reject) {
			resolve(value);
		});
	}

	var executeInChain = function(queries) {
		var when = new Promise(function (resolve, reject) {
			resolve();
		});

		var promise = queries.reduce(function(previous, query) {
		  log("chaining " + query);
		  return previous.then(function() {
		    log("executing " + query);
		    return executeSQL(DataBaseCordova.db, query, [])
		      .then(function(result) {
		        log(" done " + JSON.stringify(query));
		        return whenPromise(query);
		      });
		  });
		}, whenPromise());
		return promise;
	};

	var selectCurrentVersion = function() {
		var query = "SELECT MAX(versionNumber) AS maxVersion FROM version_history";
		var promise = executeSQL(DataBaseCordova.db, query)
		  .then(function(res) {
		    var maxVersion = res.rows.item(0).maxVersion;
		    log("Current version is " + maxVersion);
		    return maxVersion;
		  });
		return promise;
	};

	var storeVersionInHistoryTable = function(versionNumber) {
		var query = "INSERT INTO version_history (versionNumber, migratedAt) VALUES (?, ?)";
		var promise = executeSQL(DataBaseCordova.db, query, [versionNumber, new Date()])
		  .then(function(res) {
		    log("Stored version in history table: " + versionNumber);
		    return versionNumber;
		  });
		return promise;
	};

	var createVersionHistoryTable = function() {
		var query = "CREATE TABLE IF NOT EXISTS version_history(versionNumber INTEGER PRIMARY KEY NOT NULL, migratedAt DATE)";
		var promise = executeSQL(DataBaseCordova.db, query, [])
		.then(function() {
		  var versionNumber = 0;
		  return versionNumber;
		});
		return promise;
	};

	var migrate = function() {
		var initialSteps = [
		  createVersionHistoryTable,
		  selectCurrentVersion
		];

		var versions = [
			{
				versionNumber: 1,
				queries: [
					"CREATE TABLE IF NOT EXISTS CLUB_GROUPE (id INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT, cgr_no, cl_no, lc_cod, cgr_nom)",
					"CREATE TABLE IF NOT EXISTS CLUB_GROUPE_INDIVIDU (id INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT, cgr_no, in_no)",
					"CREATE TABLE IF NOT EXISTS CLUB_LICENCE (id INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT, li_no, cl_no, in_no, in_nom, in_prenom, in_sexe, lc_cod, in_dat_nais, lc_lib, sc_cod, sc_lib, li_statut, li_dat_enr, li_natu_deman, li_etat, li_mut, in_nat, ic_cachet_lib, in_photo, cq_cl_info, cg_nom, sa_lib, tl_cod, tl_lib, cl_cod, cl_nom, dpp_nom_abr, li_certif_medic, li_surclass, li_di_fonc, li_cl_grpt, in_maillot)",
					"CREATE TABLE IF NOT EXISTS EQUIPE_TYPE (id INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT, lc_cod, cl_no, cei_no, tq_no, eq_cod, sa_no)",
					"CREATE TABLE IF NOT EXISTS EQUIPE_TYPE_INDIVIDU (id INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT, li_no, cfi_poste, cfi_ordre, cfi_maillot, cfi_capi, cfi_typ_role, cfi_role, tq_no, cei_no, eq_cod, lc_cod, sa_no, cl_no, li_statut, li_mut, in_nat)",
					"CREATE TABLE IF NOT EXISTS FMI_REF (id INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT, ref_typ, ref_cod, ref_libelle, ref_sort)",
					"CREATE TABLE IF NOT EXISTS MATCH (id INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT, ma_no, sa_no, lc_cod, ma_huisclos, ma_dat, ma_heure_lib, pg_lib, ma_tv, cr_cl_no, cr_cl_cod_nom, cr_eq_no, cr_eq_cod, cr_eq_nom_abr, cr_cl_nb_maxi_eqp_typ, cr_cl_blason, cv_cl_no, cv_cl_cod_nom, cv_eq_no, cv_eq_cod, cv_eq_nom_abr, cv_cl_nb_maxi_eqp_typ, te_no, te_loca, te_nom, cp_nom, ph_lib, ph_duree_periode, ph_duree_prol, ph_prol, ph_tps_sup, ph_tir_but, ph_nb_titul, ph_nb_jou_banc, ph_nb_staff_banc, ph_nb_dir_banc, ph_nb_edu_banc, ph_nb_med_banc, ph_num_maillot, ph_tactiq, ph_reglt_locaux, ph_info_but, ph_info_secu, ph_info_orga, ph_nb_titul_mini, ph_nb_remp, ph_nbmaxi_mut, ph_nb_maxi_etrg, cfm_no, li_no, gp_lib, cfm_statut, sanctions, cv_sfl_dat_dern_trans_compo, cr_sfl_dat_dern_trans_compo, dat_dern_trans, cv_cl_blason, ma_demo)",
					"CREATE TABLE IF NOT EXISTS MATCH_FEUILLE (id INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT, cfm_no, ma_no, cfm_statut, tq_no, ma_min_period1, ma_min_period1_sup, ma_min_period2, ma_min_period2_sup, ma_min_prol, ma_min_prol_sup1, ma_min_prol_sup2, cr_forf, cv_forf, ma_min_arret, ma_min_arret_sup, ma_motif_arret, ma_motif_arret_cod, ma_motif_non_joue_cod, ma_motif_non_joue, cr_nb_but, cv_nb_but, cr_nb_tir_but, cv_nb_tir_but, cr_nb_but_tps_reglt, cv_nb_but_tps_reglt, cfm_nb_signat_avm, cfm_nb_signat_fm, in_no_avm_rec, cfm_signat_avm_rec, in_no_avm_vis, cfm_signat_avm_vis, in_no_avm_arb, cfm_signat_avm_arb, in_no_res_avm_rec, cfm_signat_res_avm_rec, in_no_res_avm_vis, cfm_signat_res_avm_vis, in_no_res_avm_arb, cfm_signat_res_avm_arb, in_no_tech_apm_rec, in_no_tech_apm_vis, in_no_tech_apm_arb, in_no_tech_apm_ass, cfm_signat_tech_apm_rec, cfm_signat_tech_apm_vis, cfm_signat_tech_apm_arb, cfm_signat_tech_apm_ass, in_no_obs_apm_rec, cfm_signat_obs_apm_rec, in_no_obs_apm_vis, cfm_signat_obs_apm_vis, in_no_obs_apm_arb, cfm_signat_obs_apm_arb, in_no_apm_rec, cfm_signat_apm_rec, in_no_apm_vis, cfm_signat_apm_vis, in_no_apm_arb, cfm_signat_apm_arb, cfm_med, cfm_tech_lum, cfm_direc_secu, cfm_ma_dat, cfm_ma_heure_lib, cfm_te_loca, cfm_te_nom, cfm_tq_diff, cfm_rapp_deleg, cfm_recette, cfm_nb_spect, cfm_reglt_locaux_lib, cfm_rapp_arb, cfm_cr_obs_apm, cfm_cv_obs_apm, cfm_res_tech_apm, chbxProlongations, chbxTirsAuBut, chbxMatchNonJoue, chbxMatchArrete, fdmValidee, observationsApresMatchNeant, reservesTechniquesNeant, cfm_verrou, cfm_mdp, cfm_infos_valid, cr_co_valid, cv_co_valid)",
					"CREATE TABLE IF NOT EXISTS MATCH_FEUILLE_COMPO (id INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT, li_no, ma_no, cfm_no, cfi_cl_no, cfi_poste, cfi_ordre, cfi_maillot, cfi_capi, cfi_typ_role, cfi_role, tq_no, li_statut, li_mut, in_nat)",
					"CREATE TABLE IF NOT EXISTS MATCH_FEUILLE_EVENEMENT (id INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT, ma_no, cfm_no, cfe_no, l1_li_no, l2_li_no, cfe_typ, cfe_min, cfe_min_sup, cfe_typ_but, cfe_action, cfe_cod_motif, cfe_lib, cfe_cod_bless, cfe_denombrement, cfe_tribune, cfe_nb_evmt, cfe_trib)",
					"CREATE TABLE IF NOT EXISTS MATCH_FEUILLE_MEMBRE (id INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT, mmf_no, ma_no, mm_no, po_cod, po_lib, mmf_typ, mmf_statut, mmf_pres, mmf_in_no, mmf_in_nom, mmf_in_prenom, cfm_no, cl_no)",
					"CREATE TABLE IF NOT EXISTS MATCH_FEUILLE_RESERVE (id INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT, mfr_no, cfm_no, cl_no, in_no, in_nom, in_prenom, in_sexe, mfr_typ, mfr_libre)",
					"CREATE TABLE IF NOT EXISTS MATCH_FEUILLE_LICENCE (id INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT, ma_no, li_no)",
					"CREATE TABLE IF NOT EXISTS SANCTION (id INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT, dd_no, in_no, li_no, dd_dat_publi, cp_nom, ma_lib, dd_lib, dd_dat_effet, ma_no)",
					"CREATE TABLE IF NOT EXISTS PARAM (id INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT, ref_typ, ref_cod, ref_libelle, ref_sort)",
					"CREATE TABLE IF NOT EXISTS TACTIQUE (id INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT, tq_no, tq_nom, tq_cod_affich)",
					"CREATE TABLE IF NOT EXISTS UTILISATEUR (id INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT, usr_no, usr_nom_conx, usr_ora, usr_password, cl_no)",
					"CREATE TABLE IF NOT EXISTS UTILISATEUR_EQUIPE (id INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT, usr_no, cl_no, eq_no, eq_cod, lc_cod, sa_no)",
					"CREATE TABLE IF NOT EXISTS TABLETTE (id INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT, tlb_uuid, tlb_cle_e, tlb_cle_b, tlb_app_version, tlb_nb_eq_typ_max, data_synchro)",
					"CREATE TABLE IF NOT EXISTS CGU (id INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT, cgu_no, usr_no, cgu_dat, cgu_version)",
					"CREATE TABLE IF NOT EXISTS FEUILLE_MATCH_HISTO (id INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT, cfmh_no, cfm_no, cfmh_dat, cfmh_heure, cfmh_flux)",
				
					"CREATE INDEX IF NOT EXISTS CLUB_GROUPE_PK                  ON CLUB_GROUPE (cgr_no);",
					"CREATE INDEX IF NOT EXISTS CLUB_GROUPE_CL_NO               ON CLUB_GROUPE (cl_no);",
					'CREATE INDEX IF NOT EXISTS CLUB_GROUPE_INDIVIDU_FK         ON CLUB_GROUPE_INDIVIDU (cgr_no);',

    				'CREATE INDEX IF NOT EXISTS CLUB_LICENCE_FK                 ON CLUB_LICENCE (li_no);',
    				'CREATE INDEX IF NOT EXISTS CLUB_LICENCE_CL_NO              ON CLUB_LICENCE (cl_no);',
    
    				'CREATE INDEX IF NOT EXISTS EQUIPE_TYPE_PK                  ON EQUIPE_TYPE (cei_no, eq_cod, lc_cod, cl_no);',
    				'CREATE INDEX IF NOT EXISTS EQUIPE_TYPE_PURGE               ON EQUIPE_TYPE (cl_no, eq_cod, sa_no, lc_cod);',
    
    				'CREATE INDEX IF NOT EXISTS EQUIPE_TYPE_INDIVIDU_PK         ON EQUIPE_TYPE_INDIVIDU (cei_no, li_no);',
    				'CREATE INDEX IF NOT EXISTS EQUIPE_TYPE_INDIVIDU_PURGE      ON EQUIPE_TYPE_INDIVIDU (cl_no, eq_cod, sa_no, lc_cod);',
    
    				'CREATE INDEX IF NOT EXISTS FMI_REF_PK                      ON FMI_REF (ref_typ, ref_cod);',
    
    				'CREATE INDEX IF NOT EXISTS MATCH_PK                        ON MATCH (ma_no);',
    
    				'CREATE INDEX IF NOT EXISTS MATCH_FEUILLE_PK                ON MATCH_FEUILLE (cfm_no);',
    				'CREATE INDEX IF NOT EXISTS MATCH_FEUILLE_MA_NO             ON MATCH_FEUILLE (ma_no);',
    
    				'CREATE INDEX IF NOT EXISTS MATCH_FEUILLE_EVENEMENT_PK      ON MATCH_FEUILLE_EVENEMENT (cfe_no);',
    				'CREATE INDEX IF NOT EXISTS MATCH_FEUILLE_EVENEMENT_CFM_NO  ON MATCH_FEUILLE_EVENEMENT (cfm_no);',
    
    				'CREATE INDEX IF NOT EXISTS MATCH_FEUILLE_LICENCE_PK        ON MATCH_FEUILLE_LICENCE (ma_no);',
    
    				'CREATE INDEX IF NOT EXISTS MATCH_FEUILLE_MEMBRE_PK         ON MATCH_FEUILLE_MEMBRE (mmf_no);',
    				'CREATE INDEX IF NOT EXISTS MATCH_FEUILLE_MEMBRE_CFM_NO     ON MATCH_FEUILLE_MEMBRE (cfm_no);',
    
    				'CREATE INDEX IF NOT EXISTS MATCH_FEUILLE_RESERVE_PK        ON MATCH_FEUILLE_RESERVE (mfr_no);',
    				'CREATE INDEX IF NOT EXISTS MATCH_FEUILLE_RESERVE_CFM_NO    ON MATCH_FEUILLE_RESERVE (cfm_no);',
    
    				'CREATE INDEX IF NOT EXISTS SANCTION_PK                     ON SANCTION (dd_no);',
    				'CREATE INDEX IF NOT EXISTS SANCTION_MA_NO                  ON SANCTION (ma_no);',
    
    				'CREATE INDEX IF NOT EXISTS TABLETTE_PK                     ON TABLETTE (tlb_uuid);',

    				'CREATE INDEX IF NOT EXISTS UTILISATEUR_USR_NO              ON UTILISATEUR (usr_no);',
    				'CREATE INDEX IF NOT EXISTS UTILISATEUR_CL_NO               ON UTILISATEUR (cl_no);',
    
    				'CREATE INDEX IF NOT EXISTS UTILISATEUR_EQUIPE_USR_NO       ON UTILISATEUR_EQUIPE (usr_no);',
    
    				'CREATE INDEX IF NOT EXISTS CGU_PK                          ON CGU (cgu_no);',
    				'CREATE INDEX IF NOT EXISTS CGU_USR_NO                      ON CGU (usr_no);',
    
    				'CREATE INDEX IF NOT EXISTS FEUILLE_MATCH_HISTO_PK          ON FEUILLE_MATCH_HISTO (cfmh_no);',
    				'CREATE INDEX IF NOT EXISTS FEUILLE_MATCH_HISTO_CFM_NO      ON FEUILLE_MATCH_HISTO (cfm_no);',

    				"ALTER TABLE MATCH_FEUILLE_COMPO ADD COLUMN usr_no;",
    				"ALTER TABLE MATCH_FEUILLE_COMPO ADD COLUMN dat_modif;",
    				"ALTER TABLE MATCH_FEUILLE_COMPO ADD COLUMN in_no;",
    				"ALTER TABLE MATCH_FEUILLE_COMPO ADD COLUMN in_nom;",
    				"ALTER TABLE MATCH_FEUILLE_COMPO ADD COLUMN in_prenom;",
    				"ALTER TABLE MATCH_FEUILLE_COMPO ADD COLUMN in_dat_nais;",

    				"ALTER TABLE MATCH_FEUILLE ADD COLUMN cfm_cr_mdp;",
    				"ALTER TABLE MATCH_FEUILLE ADD COLUMN cfm_cr_question;",
    				"ALTER TABLE MATCH_FEUILLE ADD COLUMN cfm_cr_rep;",
    				"ALTER TABLE MATCH_FEUILLE ADD COLUMN cfm_cv_mdp;",
    				"ALTER TABLE MATCH_FEUILLE ADD COLUMN cfm_cv_question;",
    				"ALTER TABLE MATCH_FEUILLE ADD COLUMN cfm_cv_rep;",
    				"ALTER TABLE MATCH_FEUILLE ADD COLUMN cfm_obs_apm;",
    				"ALTER TABLE MATCH_FEUILLE ADD COLUMN cr_abs_tech_apm;",
    				"ALTER TABLE MATCH_FEUILLE ADD COLUMN cv_abs_tech_apm;",
    				"ALTER TABLE MATCH_FEUILLE ADD COLUMN cr_abs_obs_apm;",
    				"ALTER TABLE MATCH_FEUILLE ADD COLUMN cv_abs_obs_apm;",
    				"ALTER TABLE MATCH ADD COLUMN dat_dern_sync_eq_adv;",
    				"ALTER TABLE MATCH ADD COLUMN ph_carton_blanc;",

    				"ALTER TABLE MATCH_FEUILLE_RESERVE ADD COLUMN li_no;",

    				"ALTER TABLE CLUB_LICENCE ADD COLUMN tl_uti_typ_lic;",
    				"ALTER TABLE CLUB_LICENCE ADD COLUMN li_dat_qualif;",

    				"ALTER TABLE MATCH_FEUILLE_EVENEMENT ADD COLUMN l1_cfi_maillot;",
    				"ALTER TABLE MATCH_FEUILLE_EVENEMENT ADD COLUMN l2_cfi_maillot;",
    				"ALTER TABLE MATCH_FEUILLE_EVENEMENT ADD COLUMN in_no;",
				]
			},
			{
				versionNumber: 2,
				queries: [
					"CREATE TABLE IF NOT EXISTS ERROR_LOG (id INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT, usr_no, err_msg, err_stack, err_app_nav, err_app_ctx);",
					'CREATE INDEX IF NOT EXISTS ERROR_LOG_USR_NO_FK ON ERROR_LOG (usr_no);',
					"ALTER TABLE MATCH_FEUILLE_MEMBRE ADD COLUMN mmf_typ_piece;",
					"ALTER TABLE MATCH ADD COLUMN tl_cod;",
				]
			},
			{
				versionNumber: 3,
				queries: [
					"ALTER TABLE MATCH ADD COLUMN syncunitaire;",
				]
			},
			{
				versionNumber: 4,
				queries: [
					"ALTER TABLE MATCH_FEUILLE_EVENEMENT ADD COLUMN cfe_di_hors_match;",
				]
			},
			{
				versionNumber: 5,
				queries: [
					"ALTER TABLE CLUB_GROUPE ADD COLUMN ma_no;",
					"ALTER TABLE CLUB_GROUPE_INDIVIDU ADD COLUMN ma_no;",
					"ALTER TABLE EQUIPE_TYPE ADD COLUMN ma_no;",
					"ALTER TABLE EQUIPE_TYPE_INDIVIDU ADD COLUMN ma_no;",
					"ALTER TABLE UTILISATEUR ADD COLUMN ma_no;",
					"ALTER TABLE UTILISATEUR_EQUIPE ADD COLUMN ma_no;",
				]
			},
			{
				versionNumber: 6,
				queries: [
					"ALTER TABLE MATCH ADD COLUMN dateRecuperation;",
				]
			},
			{
				versionNumber: 7,
				queries: [
					"ALTER TABLE MATCH_FEUILLE ADD COLUMN cfm_signat_res_abs_avm_rec;",
					"ALTER TABLE MATCH_FEUILLE ADD COLUMN cfm_signat_res_abs_avm_vis;",
				]
			},
			{
				versionNumber: 8,
				queries: [
    				"ALTER TABLE MATCH ADD COLUMN ph_carton_vert;"
				]
			},
			{
				versionNumber: 9,
				queries: [
    				"ALTER TABLE UTILISATEUR ADD COLUMN url;"
				]
			}
		];

		var migrationSteps = versions.map(function(version) {
		  return function(currentVersion) {
		    if (currentVersion >= version.versionNumber)
		      return whenPromise(currentVersion);

		    var promise = executeInChain(version.queries).then(function() {
		      log("Version "+version.versionNumber+" migration executed");
		      return version.versionNumber;
		    })
		    .then(storeVersionInHistoryTable);

		    return promise;
		  };
		});

		var steps = initialSteps.concat(migrationSteps);
		steps.reduce(function(current, next) {
		  return current.then(next);
		}, whenPromise())
		.then(function() {
			DataBase.dbReady = true;
		  	log("All migrations executed");
		})
		.catch(function(error) {
		  log("Error: " + JSON.stringify(error));
		});

	}

	return {
		migrate: migrate,
	};

}());

var DataBaseCordova = {
	db: null,
	dbReady: false,
	init: function() {
		console.info("DataBaseCordova.init");
		DataBase.dbReady = false;
		DataBase.db = window.sqlitePlugin.openDatabase({name: "fmi.db", location: 'default', androidLockWorkaround: 1});
//		DataBase.db = window.openDatabase(DataBaseCordova.name, "", "Umanis TODOLIST", DataBaseCordova.taille);
		DatabaseCordovaMigrationService.migrate();

		// Ajouter les nouvelles migrations de la base de données ci-dessous
		// 
		// } else if(DataBase.db.version == "NEXT VERSION") {
		// 		/* Code de la migration, ne pas oublier de modifier le code de création de la base (setup) */
		// }

		// DataBase.db.transaction(DataBase.create, DataBase.error, DataBase.success);
	},
	error: function(err, sql) {
        var message = "Error processing SQL : " + err.code + " : " + err.message;
        if (Utils.isNotNull(sql)) {
           message += " SQL : " + sql;
        }
        Engine.alert(message);
        Engine.hideLoader();
    },
    errorNonBloquante: function(err) {
        Utils.log("Error processing SQL : " + err.code + " : " + err.message);
        Engine.hideLoader();
    },
	success: function() {
        Utils.log("Connecté à la BD Cordova");
        // DataBase.db.transaction(DataBase.alterMatchFeuilleCompo, DataBase.errorNonBloquante, function() {});
        // DataBase.db.transaction(DataBase.alterMatchFeuille, DataBase.errorNonBloquante, function() {});
        // DataBase.db.transaction(DataBase.alterClubLicence, DataBase.errorNonBloquante, function() {});
        // DataBase.db.transaction(DataBase.alterMatchFeuilleReserve, DataBase.errorNonBloquante, function() {});
        // DataBase.db.transaction(DataBase.alterMatchFeuilleEvenement, DataBase.errorNonBloquante, function() {});
		DataBase.dbReady = true;
	},
	create: function(tx) {
		DataBaseCordova.createBase(tx);
	},
	/* METHODES GENERIQUES */
	transactionSqlWithSuccess: function(sql, callback, errorCallback) {
        // Engine.showLoader();
		Utils.log(sql);
		DataBase.db.transaction(function(tx) {
		    tx.executeSql(sql, [], function(tx, results) {
		    	// try { 
		    		callback(tx, results);
		    	// } catch (ex) { 
		    		// console.error(ex);
		    	 // 	Engine.alert("Erreur interne: " + ex);
		    	// } finally {
		    	// 	Engine.hideLoader();
		    	// }
			});
	    }, function (err) {
	        DataBase.error(err, sql);
			if (typeof errorCallback === 'function') {
	            errorCallback(data);
	        }
	    });
	},
	getCondition: function(table, champs, valeur) {
		var condition = " ";
		if (Utils.isNotBlank(champs)) {
    		var valeurArray = Utils.encryptData(table, valeur);
    		if(Object.prototype.toString.call(valeur) !== '[object Array]' ) {
    			valeurArray = [Utils.encryptData(table, valeur)];
    		}
    		var tokens = champs.split("And");
    		for (var i in tokens) {
    			if (valeurArray[i] != null) {
    				condition += (condition == " ") ? "where " : "and ";
    				condition += tokens[i] + " = '" + valeurArray[i] + "' ";
    			} else {
    				condition += (condition == " ") ? "where " : "and ";
    				condition += tokens[i] + " is " + valeurArray[i] + " ";
    			}
    		}
        }
		return condition;
	},
	/* FIN METHODES GENERIQUES */
	findAll: function(table, callback) {
		return new Promise(function (resolve, reject) {
			var data = [];
			var sql = "SELECT * FROM " + table + " ";
			if (DataBase.nbMax < 10000) {
			    sql += " limit 0, " + DataBase.nbMax;
			}
			DataBase.transactionSqlWithSuccess(sql, function(tx, results) {
			    for (var i=0; i< results.rows.length; i++) {
			    	data.push(Utils.decryptData(table, results.rows.item(i)));
			    }
	            if (typeof callback === 'function') {
	                callback(data);
	            }
	            resolve(data);
			}, function (err, sql) {
				reject(err, sql);
			});
		});
	},
	findById: function(table, id, callback) {
		DataBase.findBy(table, "id", id, callback);
	},
    nbMax: 10000000,
	findBy: function(table, champs, valeur, callback) {
		var data = [];
		var sql = "SELECT * FROM " + table + DataBase.getCondition(table, champs, valeur);
		if (DataBase.nbMax < 10000) {
		    sql += " limit 0, " + DataBase.nbMax;
		}
		DataBase.transactionSqlWithSuccess(sql, function(tx, results) {
		    for (var i=0; i< results.rows.length; i++) {
		    	data.push(Utils.decryptData(table, results.rows.item(i)));
		    }
            if (typeof callback === 'function') {
                callback(data);
            }
		});
	},
	selectMatchDirigeant: function(cr_cl_no, cv_cl_no, callback) {
        var result = [];
        var sql = "SELECT * FROM CLUB_LICENCE WHERE (cl_no='"+Utils.encryptData("CLUB_LICENCE", cr_cl_no)+"' OR cl_no='"+Utils.encryptData("CLUB_LICENCE", cv_cl_no)+"') GROUP BY in_no";
        if (DataBase.nbMax < 10000) {
            sql += " limit 0, " + DataBase.nbMax;
        }
        DataBase.transactionSqlWithSuccess(sql, function(tx, results) {
            for (var i=0; i< results.rows.length; i++) {
                result.push(Utils.decryptData("CLUB_LICENCE", results.rows.item(i)));
            }
            if (typeof callback === 'function') {
                callback(result);
            }
        });
	},
	countBy: function(table, champs, valeur, callback) {
		var sql = "SELECT * FROM " + table + DataBase.getCondition(table, champs, valeur);
		DataBase.transactionSqlWithSuccess(sql, function(tx, results) {
            if (typeof callback === 'function') {
    			callback(results.rows.length);
            }
		});
	},
	insert: function(table, data, callback) {
		var batch = []
		if (data.constructor == Array) {                                
			for (var i in data) {
				var encryptData = Utils.encryptData(table, data[i]);
				var champs = "(";
				var values = "(";
				for(var key in encryptData){
					champs += key + ",";
					if(encryptData[key] === null){
						values += "null,";
					}else{
						values += "'" + encryptData[key] + "',";
					}
				}
				champs = champs.substring(0, champs.length - 1) + ")";
				values = values.substring(0, values.length - 1) + ")";
				var sql = "INSERT INTO " + table + " " + champs + " values " + values;                                                
				batch.push(sql);
			}
			//console.log(batch);
			DataBase.db.sqlBatch(batch, function() {
				if (typeof callback === 'function') {
					callback();
				}
			});                                         
		} else {
			var encryptData = Utils.encryptData(table, data);
			var champs = "(";
			var values = "(";
			for(var key in encryptData){
				champs += key + ",";
				if(encryptData[key] === null){
					values += "null,";
				}else{
					values += "'" + encryptData[key] + "',";
				}
			}
			champs = champs.substring(0, champs.length - 1) + ")";
			values = values.substring(0, values.length - 1) + ")";
			var sql = "INSERT INTO " + table + " " + champs + " values " + values;
			batch.push(sql);                                             
			DataBase.db.sqlBatch(batch, function() {
				if (typeof callback === 'function') {
					callback();
				}
			}); 
		}                  
/*	    var encryptData = Utils.encryptData(table, data);
		var champs = "(";
		var values = "(";
		for(var key in encryptData){
			champs += key + ",";
			if(encryptData[key] === null){
				values += "null,";
			}else{
				values += "'" + encryptData[key] + "',";
			}
			
		}
		champs = champs.substring(0, champs.length - 1) + ")";
		values = values.substring(0, values.length - 1) + ")";
		var sql = "INSERT INTO " + table + " " + champs + " values " + values;
		DataBase.transactionSqlWithSuccess(sql, function(tx, results) {
            if (typeof callback === 'function') {
                callback();
            }
		});*/
	},
	update: function(table, id, champs, valeur, callback) {
		var value = Utils.encryptData(table, valeur);
		var sqlValue = value !== null ? ("'"+value+"'") : null;
		var sql = "UPDATE " + table + " set " + champs + " = " + sqlValue + " where id = '" + id + "'";
		DataBase.transactionSqlWithSuccess(sql, function(tx, results) {
            if (typeof callback === 'function') {
                callback();
            }
		});
	},
    updateJson: function(table, id, json, callback) {
        DataBase.findById(table, id, function(result) {
            var update = "";
            for (var i in json) {
                if (i != "id") {
                	if (json[i] !== null) {
                		update += i + " = '" + Utils.encryptData(table, json[i]) + "', ";
                	} else {
                		update += i + " = null, ";
                	}
                    
                }
            }
            if (update.length > 0) {
                update = update.substring(0, update.length - 2);
            }
            Utils.log(update);
            if(Utils.isNotBlank(update)) {
                var sql = "UPDATE " + table + " set " + update + " where id = '" + id + "'";
                DataBase.transactionSqlWithSuccess(sql, function(tx, results) {
                    if (typeof callback === 'function') {
                        callback();
                    }
                });
            } else {
                if (typeof callback === 'function') {
                    callback();
                }
            }
        });
    },
	deleteById: function(table, id, callback) {
		var sql = "DELETE FROM " + table + " where id = " + id;
		DataBase.transactionSqlWithSuccess(sql, function(tx, results) {
            if (typeof callback === 'function') {
                callback();
            }
		});
	}
};