/**
 * Classe Engine : moteur du Framework.
 */
var Engine = {
	modules: new Array(),
	tailleDecoupageFlux: 32000,
    loadHeadJs: true,
	mainPage: null,
	isMobile: null,
	isCordova: null,
	isWindows: null,
	dbSiteWeb: true,
	dbActif: false,
	indexedDbBd: false,
	eventClick: "click",
    eventDblclick: null,
	testerCompatibiliteIndexedDb: true,
	userAgentIE: /IE/i,
	userAgentMobile: /Android|webOS|iPhone|iPad|iPod|BlackBerry|Opera Mini|IEMobile/i,
	errorIndexedDb: "Votre navigateur n'est pas compatible! Merci d'utiliser Firefox ou Chrome.",
	idProgressBar: "#progressbar",
	device: {
        "name": "MODE_WEB",
        "cordova": "",
        "platform": "",
        "uuid": "0",
        "model": "",
        "version": "",
    },
    filAriane: [],
    scrolls : {},       // Array pour les zones scrolls
    /**
     * Retourne true si l'application courante est
     * en mode mobile.
     */
    isCordovaMobile: function () {
        var matchUserAgent = navigator.userAgent.match(Engine.userAgentMobile);
        return Engine.isCordova || (matchUserAgent !== null && matchUserAgent.length > 0 && matchUserAgent[0] !== "");
    },
	/**
	 * Initialisation du moteur. 
	 */
	init: function() {
        Utils.log("=================================================");
		Utils.log("INITIALISATION DE l'ENGINE");
		Engine.detectionSupport();
	},

     /**
     * Initialise tous les scrolls de la page en cours.
     * @param {int} idScroll id de la zone scrollable
     */
    initScrolls: function(idScroll){
        if (Engine.isCordovaMobile()) {
            // Définir un scroll spécifique 
            if(typeof idScroll !== 'undefined') {
                if(Engine.scrolls[idScroll] !== undefined){
                    Engine.deleteScroll[idScroll];
                }
                if($("#" + idScroll).hasClass("divScrollable")) {
                     Engine.scrolls[idScroll] = new ScrollView(idScroll, false, true);
                 } else if ($("#" + idScroll).hasClass("divScrollableHorizontally")) {
                     Engine.scrolls[idScroll] = new ScrollView(idScroll, true, false);
                 }
            } else {  
                // Définir des scrolls pour tous les composants scrollables sur la page 
                $('.divScrollable').each(function(){
                    var id = $(this).attr('id');
                    if (Engine.scrolls[id] === undefined) {
                        Engine.scrolls[id] = new ScrollView(id, false, true);
                    } else {
                        Engine.scrolls[id].refresh();
                    }
                });
                // Les scrolls horizontals 
                $('.divScrollableHorizontally').each(function(){
                    var id = $(this).attr('id');
                    if (Engine.scrolls[id] === undefined) {
                        Engine.scrolls[id] = new ScrollView(id, true, false);
                    } else {
                        Engine.scrolls[id].refresh();
                    }
                });
            }
        }
    },
    /**
    * Supprimer les deux div autour de cette zone scrollable et cet objet dans le map
    * @param {int} idScroll id de la zone scrollable
    */ 
    deleteScroll : function(idScroll) {
         if (Engine.isCordovaMobile()) {
            if(typeof Engine.scrolls[idScroll] !== 'undefined') {
                Engine.scrolls[idScroll].destroy();
                delete Engine.scrolls[idScroll];
            } else if($("#" + idScroll).find(".divScrollable").length >= 1 || 
                $("#" + idScroll).find(".divScrollableHorizontally").length >= 1 ) {

                    $("#" + idScroll).find(".divScrollable").each(function(cle, valeur){
                        var idValeur = valeur.id;
                        if(typeof Engine.scrolls[idValeur] !== 'undefined') {
                            Engine.scrolls[idValeur].destroy();
                            delete Engine.scrolls[idValeur];
                        } 
                    });
                    // Les scrolls horizontals
                    $("#" + idScroll).find(".divScrollableHorizontally").each(function(cle, valeur){
                        var idValeur = valeur.id;
                        if(typeof Engine.scrolls[idValeur] !== 'undefined') {
                            Engine.scrolls[idValeur].destroy();
                            delete Engine.scrolls[idValeur];
                        } 
                    });
            }
        }
    },
    /**
     * Fonction permettant de charger les fichiers JS lié à la gestion de la persistance.
     * 
     * @param {List<String>} fichiers Les fichiers à charger
     * @param {Boolean} cordovaBd Charger la base de données de Cordova ?
     * @param {Boolean} indexedDbBd Charger la base de données de IndexedDb ?
     *  
     */
	headInit: function(fichiers, cordovaBd, indexedDbBd) {
	    var fichier = fichiers.shift();
	    if (Engine.loadHeadJs) {
            head.js(
                fichier,
                function() {
                    if (fichiers.length == 0) {
                        if (cordovaBd) {
                            DataBase = DataBaseCordova;
                            DataBaseIndexedDb = {};
                            DataBaseWeb = {};
                        } else if (indexedDbBd) {
                            DataBase = DataBaseIndexedDb;
                            DataBaseCordova = {};
                            DataBaseWeb = {};
                        } else {
                            DataBase = DataBaseWeb;
                            DataBaseCordova = {};
                            DataBaseIndexedDb = {};
                        }
                        Engine.chargementPages();
                    } else {
                        Engine.headInit(fichiers, cordovaBd, indexedDbBd);
                    }
                }
            );
        } else {
            DataBase = DataBaseIndexedDb;
            Engine.chargementPages();
        } 
	},	
	/**
	 * Détection du support.
	 */
	detectionSupport: function() {
		// détection du support
		
		var isIeMobile = false;
        if (Environnement.cordovaPlateforme !== "browser") {
            isIeMobile = navigator.userAgent.match(this.userAgentIE);
            isIeMobile = isIeMobile || !!navigator.userAgent.match(/Trident.*rv[ :]*11\./);
        }
		
        var matchUserAgent = navigator.userAgent.match(Engine.userAgentMobile);

		this.isMobile = matchUserAgent !== null && matchUserAgent.length > 0 && matchUserAgent[0] !== "";

        // En mode développement, on doit autoriser la création d'une base de données
        // sur un navigateur
        if (!this.isMobile && Environnement.cordovaPlateforme !== 'browser') {
            Engine.dbSiteWeb = true;
        } else {
            Engine.dbSiteWeb = false;
        }

		// évènement à binder pour le clique (PC) ou au doigt (mobilité)
		this.eventClick = "click";
		this.eventDblclick = this.isMobile ? "touchend" : "dblclick";
		
		var cordovaBd = false;
		this.indexedDbBd = false;
		this.isWindows = false;

		// 1er cas : application mobile sous Android ou iOS
		if (this.isMobile && !isIeMobile && Environnement.cordovaPlateforme !== 'browser') {
			this.isCordova = true;
            this.isMobile = true;
            cordovaBd = true;
		} else if (isIeMobile && Environnement.cordovaPlateforme !== 'browser') {
			// 2eme cas : application mobile sous Windows Mobile
			this.isCordova = true;
			this.isMobile = true;
            this.indexedDbBd = true;
            this.isWindows = true;
            var oldAppendChild = HTMLElement.prototype.appendChild;
            HTMLElement.prototype.appendChild = function () {
                var self = this;
                var args = arguments;
                return MSApp.execUnsafeLocalFunction(function () {
                    return oldAppendChild.apply(self, args);
                });
            };
		} else {
			// 3eme cas : site Web
			this.isCordova = false;
            this.indexedDbBd = Engine.dbSiteWeb;
		}
		Engine.dbActif = Engine.dbSiteWeb || Engine.isCordova; 
		
        // gestion de la fentre pour tout le monde
        var height = $(document).height();
        if (!Engine.isCordova && parseInt(height, 10) < 750) {
            height = 750;
        }
        $(".ui-popup-screen").height(height);
        $("body").height(height);
        $("#mainContent").height(height);
        $("#mainContent").css("max-height", height);
            
		// chargement des fichiers JS pour Cordova et la persistance
        var fichiers = [];
        if (this.isCordova) {
            fichiers.push("cordova.js");
        }
        
        if (cordovaBd) {
            fichiers.push("js/fwk/DataBaseCordova.js");
//            fichiers.push("modules/technique/CreateBaseCordova.js");
        } else if (this.indexedDbBd) {
            fichiers.push("js/fwk/DataBaseIndexedDb.js");
            fichiers.push("modules/technique/CreateBaseIndexedDb.js");
        } else {
            fichiers.push("js/fwk/DataBaseWeb.js");
            fichiers.push("modules/technique/CreateBaseWeb.js");
        }
        head.ready(document, function () {
            Engine.headInit(fichiers, cordovaBd, Engine.indexedDbBd);
        });
	},
	/**
	 *  Chargement des modules.
	 */
	chargementPages: function() {
        Utils.log("WIDTH ECRAN : " + $(window).width());
        Utils.log("HEIGHT ECRAN : " + $(window).height());
        if (Engine.isCordova) {
    	    if (($(window).width() < 600)) {
    	        $("#mainContent div:first").html(Lang.tabletteUniquement);
    	        $("#mainContent div:first").attr("style", "background: none repeat scroll 0 0 #FFFFFF; color: #FF0000; margin: 109px 0 0 13px; position: absolute; text-align: center; width: 96%;");
    	        return;
    	    }
        } else if (!Engine.dbSiteWeb) {
            this.modules = Project.modulesWeb;
        }
	    
        if (Engine.loadHeadJs) {
    		var go = false;
    		for(var i in this.modules) {
    			var module = "modules/" + Engine.modules[i];
    			head.js(
    				module,
    				function() {
    					if (i==Engine.modules.length-1 && !go) {
    						go = true;
    						Engine.startApplication();
    					}
    				}
    			);
    		}
		} else {
            Engine.startApplication();
		}
	},
	/**
	 * Démarrage de l'application. 
	 */
	startApplication: function() {
	    // sous Cordova (application mobile) : la base de données est initialisée quand l'application est dispo
		if (this.isCordova) {
            document.addEventListener("deviceready", function() {
                Engine.finishInit();
            }, false);
		} else {
			// ATTENTION on test la compatibilité d'IndexedDb
			var go  = true;
            var myNav = navigator.userAgent.toLowerCase();
			if (myNav.indexOf('msie') != -1 && Utils.isIe() <= 9) {
				go = false;
			}
			window.indexedDB = window.indexedDB || window.mozIndexedDB || window.webkitIndexedDB || window.msIndexedDB;
			window.IDBTransaction = window.IDBTransaction || window.webkitIDBTransaction || window.msIDBTransaction;
			window.IDBKeyRange = window.IDBKeyRange || window.webkitIDBKeyRange || window.msIDBKeyRange;
			if ((!window.indexedDB || !go) && this.testerCompatibiliteIndexedDb) {
				$(this.idProgressBar).html(this.errorIndexedDb);
			} else {
		    	Engine.finishInit();
			}
		}
	},
	/**
	 * Finalisation de l'initialisation. 
	 */
	headerController: null,
	synchronisationController: null,
	evenementsGeneriquesController: null,
	batterieLevel: null,
	isPlugged: true,
	reseauConnexion: null,
	finishInit: function() {
		try {
	    	DataBase.init();
		} catch (ex) {
			Engine.alert("Erreur initialisation de la BD : " + ex);
		}
		var sync = new Async();
		sync.start(function() {
			if (DataBase.dbReady) {
                Utils.log("FIN  DE L'INITIALISATION DE l'ENGINE");
                Utils.log("=================================================");
				sync.stop();

				if (Engine.isCordova) {
    				document.addEventListener("deviceready", function() {
                        //
                        // FEV14 EVO 4 (FMH)
                        // -----------------
                        // Empêcher le zoom sur le texte de l'application depuis les préférences système
                        // 
                        if (window.MobileAccessibility) {
                            window.MobileAccessibility.usePreferredTextZoom(false);
                        }

                        if (window.device) {
                            Engine.device = {
                                "name": Utils.isNotNull(device.name) ? device.name : Utils.isNotNull(device.model) ? device.model : "",
                                "cordova": Utils.isNotNull(device.cordova) ? device.cordova : "",
                                "platform": Utils.isNotNull(device.platform) ? device.platform : "",
                                "uuid": Utils.isNotNull(device.uuid) ? device.uuid.toLowerCase() : "0",
                                "model": Utils.isNotNull(device.model) ? device.model : "",
                                "version": Utils.isNotNull(device.version) ? device.version : ""
                            };
                        }

                        //
                        // FEV13 EVO 7 (FMH)
                        // -----------------
                        // Gestion du back button sur android :
                        // Si on appui sur le bouton back de l'appareil, on affiche une popup de confirmation
                        // avant de quitter l'application
                        // 
                        document.addEventListener("backbutton", function(event) {
                            event.preventDefault();
                            var secondPopupLevel = $("#popup-gen").length > 0;

                            Engine.addView("#warningConfirmPopup","index.html:warningConfirmPopup", { data : {texte: Lang.confirmQuitter} });
                            Engine.popup("#warningConfirmPopup", secondPopupLevel);
                            $("#warningConfirmPopupOk").click(function(e) {
                                e.stopPropagation();
                                e.preventDefault();
                                Engine.fermerPopup(secondPopupLevel);
                                if (navigator.app !== undefined) {
                                    navigator.app.exitApp();
                                }
                            });
                            $("#warningConfirmPopupCancel").click(function(e) {
                                e.stopPropagation();
                                e.preventDefault();
                                Engine.fermerPopup(secondPopupLevel);
                            });
                        }, false);
    				    
                        window.addEventListener("batterystatus", onBatteryStatus, false);
                        window.addEventListener("batterylow", onBatteryStatus, false);
                        window.addEventListener("batterycritical", onBatteryStatus, false);
                        window.addEventListener("offline", onConnectionChange, false);
                        window.addEventListener("online", onConnectionChange, false);
                        
                        function onBatteryStatus(info) {
                            Engine.batterieLevel = info.level;
                            Engine.isPlugged = info.isPlugged;
                            if (Utils.isNotNull(Engine.headerController)) {
                                Engine.headerController.showBatterie();
                            }
                        }
              
                        function onConnectionChange() {
                            var networkState = navigator.connection.type;
                            var states = {};
                            states[Connection.UNKNOWN] = 'Unknown connection';
                            states[Connection.ETHERNET] = 'Ethernet connection';
                            states[Connection.WIFI] = 'WiFi connection';
                            states[Connection.CELL_2G] = 'Cell 2G connection';
                            states[Connection.CELL_3G] = 'Cell 3G connection';
                            states[Connection.CELL_4G] = 'Cell 4G connection';
                            states[Connection.CELL] = 'Cell generic connection';
                            states[Connection.NONE] = 'No network connection';
                            Engine.reseauConnexion = states[networkState];
                            if (Utils.isNotNull(Engine.headerController)) {
                                Engine.headerController.showConnexionStatus();
                            }
                        }
                        
                        if (navigator.connection) {
                            onConnectionChange();
                        }
                        
    				}, false);
				}

                Engine.evenementsGeneriquesController = new EvenementsGeneriquesController("#page");
                Engine.headerController = new HeaderController("#page");
                Engine.headerController.load("header", [], function() {
                    if (Engine.dbActif) {
                        Engine.synchronisationController = new SynchronisationController("#page");
                        Engine.synchronisationController.load("synchronisation", [], function() {
                            Engine.gotoPage(Engine.mainPage);
                        });
                    } else {
                        Engine.gotoPage(Engine.mainPage);
                    }
                }); 
			}
		});
	},
	isBatteryOk: function() {
        return Environnement.debug === true || Engine.batterieLevel == null || Engine.isPlugged === true ||
                (Environnement.debug === false && Engine.batterieLevel != null && Engine.batterieLevel > 10);                            
    },
	/**
	 * Aller à une page.
	 * 
	 * @param {String} page La page demandé (fichier.html)
	 * @param {Json} data Les données à transmettre à la page
	 */
	pagesLoad: [],
	filesLoad: [],
    controllers: {},
    controllerActif: null,
	gotoPage: function(page, data) {
	    // listes.html = l'object listesController
	    var pageName = page.replace("#", "").replace(".html", "");
		var controllerName = Utils.capitalize(pageName) + "Controller";
		// on cherche si le Controller a déjà été instancié
		
		if (Utils.isNull(Engine.controllers[controllerName])) {
		    // Pour une première instanciation : on demande la création de ce dernier que l'on positionne dans la variable de classe "controllers"
		    Utils.log("ENGINE - CREATION DU CONTROLLER : " + controllerName);
            eval('Engine.controllers.' + controllerName + ' = (new ' + controllerName + '("#page"))');
            Engine.controllers[controllerName].name = "#" + controllerName;
            Engine.controllers[controllerName].page = page;
        } else {
		    Utils.log("ENGINE - CONTROLLER EXISTANT : " + controllerName);
        }
        Engine.controllerActif =  Engine.controllers[controllerName];
        // sauvegarde de la page précédente
        Engine.filAriane.push(page);

        // Destruction et reinitialisation des scrolls
        for(var i in Engine.scrolls) {
             Engine.scrolls[i].destroy();
        }
        Engine.scrolls = {};

        // appel de la méthode "load" du controller connu
        Engine.controllers[controllerName].load(page, data);
	},
	/**
	 * Chargement d'un Controller : chargement des fichiers JS et CSS puis de la page HTML et redirection vers la pages HTML via CanJs.
	 * 
     * @param {String} page La page courante (ex: index.html)
     * @param {List} pages La liste des pages HTML à charger
     * @param {List} files La liste des fichiers JS et CSS à charger
     * @param {controller} controller Le controller qui vient d'appeler la méthode
     * @param {Function} callback La fonction de callBack
	 */
	loadController: function(page, pages, files, controller, callback) {
        try {
            if (!Engine.loadHeadJs) {
                Engine.loadControllerHtmlFile(page, pages, controller, callback);
            } else if (files.constructor == Array || files.constructor == Object) {
                // Chargement des fichiers JS et CSS
                var filesToLoad = [];
                var filesCharge = [];
                // on parcours la liste des fichiers
                for (i in files) {
                    // si le fichier n'a pas été chargé
                    if ($.inArray(files[i], this.filesLoad) == -1) {
                        filesToLoad.push(files[i]);
                    } else {
                        // sinon on le met dans les fichiers chargés !
                        filesCharge.push(files[i]);
                    }
                }
                // s'il y a des fichier à charger
                if (filesToLoad.length > 0) {
                    Utils.log("=====> Chargement des fichiers JS & CSS de " + page + " : " + filesToLoad);
                    if (filesCharge.length > 0) {
                        Utils.log("==========> Fichier JS & CSS déjà chargé : " + filesCharge);
                    }
                    var go = false;
                    // le chemin par défaut est celui de la page courante - qui ne doit pas bouger...
                    var href = window.parent.location.href.replace("/index.html", "");
                    for(var i in filesToLoad) {
                        // on les positionne comme étant chargé !
                        this.filesLoad.push(filesToLoad[i]);
                        head.js(
                            href + "/modules/" + filesToLoad[i],
                            function() {
                                if (filesToLoad.length + filesCharge.length == files.length && !go && i == filesToLoad.length-1) {
                                    go = true;
                                    Engine.loadControllerHtmlFile(page, pages, controller, callback);
                                }
                            }
                        );
                    }
                } else {
                    Utils.log("=====> Fichier JS & CSS déjà chargé pour " + page + " : " + files);
                    Engine.loadControllerHtmlFile(page, pages, controller, callback);
                }
            } else {
                Utils.log("=====> Pas de fichiers JS & CSS à charger pour " + page);
                Engine.loadControllerHtmlFile(page, pages, controller, callback);
            }
        } catch(ex) {
            Utils.error(ex);
        }
	},
	/**
	 * Chargement des fichiers HTML du controlleur. 
	 * 
     * @param {String} page La page courante (ex: index.html)
     * @param {List} pages La liste des pages HTML à charger
     * @param {can.Control} controller Le controller qui vient d'appeler la méthode
     * @param {Function} callback La fonction de callBack
	 */
	loadControllerHtmlFile: function(page, pages, controller, callback) {
        if ((pages.constructor == Array || pages.constructor == Object) && pages.length > 0) {
            // Chargement des fichiers HTML
            var pagesToLoad = [];
            var pagesCharge = [];
            // on parcours la liste des fichiers
            for (i in pages) {
                // si le fichier n'a pas été chargé
                if ($.inArray(pages[i], this.pagesLoad) == -1) {
                    pagesToLoad.push(pages[i]);
                } else {
                    // sinon on le met dans les fichiers chargés !
                    pagesCharge.push(pages[i]);
                }
            }
            // s'il y a des fichier à charger
            if (pagesToLoad.length > 0) {
                Utils.log("=====> Chargement des fichiers HTML de " + page + " : " + pagesToLoad);
                if (pagesCharge.length > 0) {
                    Utils.log("==========> Fichier HTML déjà chargé : " + pagesCharge);
                }
                var go = false;
                for(var i in pagesToLoad) {
                    Engine.pagesLoad.push(pagesToLoad[i]);
                    var href = window.parent.location.href.replace("/index.html", "");
                    $.ajax({
                        url : href + "/modules/" + pagesToLoad[i],
                        type : "GET",
                        dataType : "html",
                        async: false,
                        success : function(xhr) {
                            $("body").append(xhr);
                            if (i == pagesToLoad.length - 1 && !go) {
                                go = true;
                                setTimeout(function() { Engine.changePage(page, controller, callback); }, 0);
                            }
                        }
                    });
                }
            } else {
                Utils.log("=====> Fichier HTML déjà chargé pour " + page + " : " + pages);
                Engine.changePage(page, controller, callback);
            }
        } else {
            Utils.log("=====> Pas de fichiers HTML à charger pour " + page);
            Engine.changePage(page, controller, callback);
        }
	},
    /**
     * Changement de page via CanJs
     * 
     * @param {String} page La page courante (ex: index.html)
     * @param {can.Control} controller Le controller qui vient d'appeler la méthode
     * @param {Function} callback La fonction de retour
     */
	changePage: function(page, controller, callback) {
	    if (page != undefined && page != "" && page.indexOf(".html") != -1) {
            Engine.fermerPopup();
            var pageName = page.replace("#", "").replace(".html", "");
            var controllerName = Utils.capitalize(pageName) + "Controller";
            
            $("#mainContent").html("");
            if (Engine.controllerActif.options.header != false) {
                $("#mainContent").html("<div id='header'></div>");
                Engine.addView("#mainContent #header", "header.html", {}, false, true);
                Engine.headerController.showLogo();
                Engine.headerController.showUtilisateurSynchro();
                Engine.headerController.showButton(page);
                Engine.headerController.showEquipes(page);
                Engine.headerController.showBatterie();
                Engine.headerController.showConnexionStatus();
            }
            
            Engine.addView("#mainContent", page.replace("#",""), {}, false, true);
            //$("#mainContent").append(Engine.addView(page.replace("#",""), {}));
            $('#mainContent').attr("class", controllerName);
            if (Utils.isNotBlank(Engine.login)) {
                $('#mainContent').addClass("connecte");
            }
            
            $('#mainContent').find(".popup").hide();
            $('#mainContent').find("input:text, input:password").each(function() {
                if(!$(this).hasClass("select2-focusser") && !$(this).hasClass("dont-touch")) {
                    var html = "<div class='ui-input-text ui-shadow-inset'>" + $(this)[0].outerHTML; 
                    $(this).before(html);
                    $(this).remove();
                }
            });
            Engine.bouton("#mainContent");
            
            // 
            // FMH - 0034747 - Ecran faits de match : apparition d'écran bleu
            // Différer le chargement du contrôleur pour éviter le chargement d'un objet référencé 
            // dans un fichier qui n'aurai pas eu le temps d'être chargé.
            // 
            setTimeout(function () {
                callback(controller);
            },200);
            
            if (Engine.controllerActif.options.all != true) {
                if (Engine.login == "") {
                    Engine.gotoPage("connexion.html");
                }
            }
       } else {
            // 
            // FMH - 0034747 - Ecran faits de match : apparition d'écran bleu
            // Différer le chargement du contrôleur pour éviter le chargement d'un objet référencé 
            // dans un fichier qui n'aurai pas eu le temps d'être chargé.
            // 
            setTimeout(function () {
                callback(controller);
            },200);
       }
	},
	sync: null,
	addView: function(id, page, data, popup, append) {
	    var data = (Utils.isNotNull(data) ? data : {}); 
	    var popup = (Utils.isNotNull(popup) ? popup : false); 
	    var append = (Utils.isNotNull(append) ? append : false); 
        data["lang"] = Lang;
        try {
            var view = can.view(page, data);
        } catch(exception) {
            Utils.error(exception);
        }
        if (append) {
            $(id).append(view);                       
        } else {
            $(id).html(view);
        }
        
        if (Utils.isNotNull(Engine.sync)) {
            Engine.sync.stop();
        }

        Engine.bouton(id);

        /*Intégration d'IScroll*/
        setTimeout(function () {
            Engine.initScrolls();
            Engine.updateDeprecatedUScroll();
        }, 100);

        return view;
    },

    /**
    * Update zone scrollable 
    **/
    updateUScrollView: function () {
        if (Engine.isCordovaMobile()) {
            for(var i in Engine.scrolls) {
                Engine.scrolls[i].refresh();
            }
        }
    },

    updateDeprecatedUScroll: function () {
        if (Engine.isCordovaMobile()) {
            $('#mainContent').find(".divScrollableUscroll").each(function () {
                $(this).uScroll({ horizontal: false });
            });
           
            $('#mainContent').find(".divScrollableHorizontallyUscroll").each(function () {
                $(this).uScroll({ horizontal: true });
            });
        }
    },

	bouton: function(id) {
	    $(id).find("a.button").each(function(){
             var button = "<span ";
            
            //ajout de tous les attributs du <a> vers le <span>
            $.each(this.attributes, function() {
                if(this.specified) {
                    button += this.name + "='" + this.value + "' ";
                }
            });
            button += ">";
            if ($(this).attr('data-type') !== undefined && $(this).attr('data-type') == "img") {
                button += "<span class='span_button_icon " + $(this).attr("data-icon") + "'></span><span class='span_button_text'>" + $(this).html() + "</span></span>";
            }
            else {
                button += "<span class='span_button_content'><span>" + $(this).text() + "</span></span></span>";
            }
            $(this).replaceWith(Engine.isWindows ? toStaticHTML(button) : button);
        });
	},
    _loadingCallStackCount: 0,
    /**
     * Afficher le spinner
     */
    showLoader: function() {
        this._loadingCallStackCount = this._loadingCallStackCount + 1;
        $(".ajax-loader").show();
        $("#div-ajax-cacher").show();
    },
    /**
     * Cacher le spinner
     */
    hideLoader: function() {
        if (this._loadingCallStackCount > 0) {
            this._loadingCallStackCount = this._loadingCallStackCount - 1;
        }
        var hideAsync = can.proxy(function() {
            if (this._loadingCallStackCount === 0) {
                $(".ajax-loader").hide();
                $("#div-ajax-cacher").hide();
            }
        }, this);
        setTimeout(hideAsync, 500);
    },
	/**
	 * Délégation des actions des controlleur.
	 * 
     * @param {can.Control} controller Le controlleur courrant
     * @param {Object} element L'élément du controlleur
     * @param {Object} event l'évènement du controlleur
     * @param {Object} callback La fonction de retour
	 */
	delayMasque: 100,
	delegate: function(controller, element, event, callback, delaiCustom) {
	    var delai = Utils.isNotNull(delaiCustom) ? delaiCustom : this.delayMasque;
	    Engine.masqueAntiClick(delai, function(){
	        // try {
                callback(controller, element, event);
            // } catch(ex) {
                // Utils.error(ex);
            // }
	    });
	},
	masqueAntiClick: function(temps, callback) {
	    // application d'un masque pour bloquer les actions de l'utilisateur X temps
        $("#div-cacher").show();
        // try catch de l'action du controlleur
        callback();
        // désactivation du masque
        setTimeout(function() { $("#div-cacher").hide(); }, temps);
	},
	action: null,
    dbclick: function(controller, element, event, callback) {
        if (Engine.isMobile) {
            var now = new Date().getTime();
            var lastTouch = element.data('lastTouch') || now + 1;
            var delta = now - lastTouch;
            clearTimeout(Engine.action);
            if(delta < 500 && delta >0) {
                Engine.delegate(controller, element, event, callback);
           } else {
                element.data('lastTouch', now);
                Engine.action = setTimeout(function(e) { clearTimeout(Engine.action); }, 500, [event]);
           }
           element.data('lastTouch', now);
        } else {
           Engine.delegate(controller, element, event, callback);
        }
    },
    fermerPopup: function(deuxiemePopup) {
        if (Utils.isNull(deuxiemePopup) || !deuxiemePopup || deuxiemePopup == "false") {
            $("#mainContent>div").unbind('touchmove');
        }
        var idPopupSuperieur = (Utils.isNotNull(deuxiemePopup) && (deuxiemePopup === 'true' || deuxiemePopup === true)) ? "-superieure" : "";
        $("#div-popup" + idPopupSuperieur).remove();
        $("#popup-gen" + idPopupSuperieur).remove();
    },
    /**
     * Ouverture d'une popup.
     * 
     * @param {Object} idPopup L'id de l'element à insérer dans la popup
     * @param {Object} deuxiemePopup Booleen (par défaut à false) indiquant si la popup est à afficher par dessus une première popup
     */
    popup: function(idPopup, deuxiemePopup) {
        deuxiemePopup = Utils.isNotNull(deuxiemePopup) ? deuxiemePopup : false; 
        var idPopupSuperieur = deuxiemePopup ? "-superieure" : "";
        Engine.fermerPopup(deuxiemePopup);
        var width = $("#mainContent").width();
        var height = $("#mainContent").height();
        
        var divPopup = "<div id='div-popup" + idPopupSuperieur + "' data-superieure='" + deuxiemePopup + "'></div>";
        if(deuxiemePopup) {
            $("#popup-gen").after(Engine.isWindows ? toStaticHTML(divPopup) : divPopup);
        }
        else {
            $("#mainContent").prepend(Engine.isWindows ? toStaticHTML(divPopup) : divPopup);
        }
        $("#div-popup" + idPopupSuperieur).width(width);
        $("#div-popup" + idPopupSuperieur).height(height);
        $("#div-popup" + idPopupSuperieur).show();
        $("#div-popup" + idPopupSuperieur).addClass("show");
        
        var htmlPopup = "<div id='popup-gen" + idPopupSuperieur + "' data-superieure='" + deuxiemePopup + "'>";
        if (!$(idPopup).hasClass("dont-close")) {
            htmlPopup += "<div class='croixFermeturePopup'></div>";
        }
        htmlPopup += "<div class='contentPopup'></div></div>";
        if(deuxiemePopup) {
            $("#div-popup-superieure").after(Engine.isWindows ? toStaticHTML(htmlPopup) : htmlPopup);
        }
        else {
            $("#div-popup").after(Engine.isWindows ? toStaticHTML(htmlPopup) : htmlPopup);
        }

        $("#popup-gen" + idPopupSuperieur + " .contentPopup").html($(idPopup).html());
        setTimeout(function () {
            $("#popup-gen" + idPopupSuperieur).addClass("show");
        }, 100);

        $("#popup-gen" + idPopupSuperieur).css("margin-left", ((width - $("#popup-gen" + idPopupSuperieur).width()) / 2) + "px");
        $("#popup-gen" + idPopupSuperieur).css("margin-top", ((height - $("#popup-gen" + idPopupSuperieur).height()) / 2) + "px");
        if ($(idPopup).hasClass("dont-close")) {
            $("#popup-gen" + idPopupSuperieur).addClass("dont-close");
        } else {
            $("#popup-gen" + idPopupSuperieur).removeClass("dont-close");
        }
        var style = $(idPopup).attr("style").split(";");
        for (var i in style) {
            var tokens = style[i].split(":"); 
            $("#popup-gen" + idPopupSuperieur).css($.trim(tokens[0]), tokens[1]);
        }
        $("#popup-gen" + idPopupSuperieur).show();
        
        Engine.selectCreate($("#popup-gen" + idPopupSuperieur).find("select"), { minimumResultsForSearch : -1 });
        

        // Resinitialiser les zones scrollables  
        if ($("#popup-gen" + idPopupSuperieur).find(".divScrollable").length != 0) {
            $("#popup-gen" + idPopupSuperieur).find(".divScrollable").each(function(cle, value){
                Engine.deleteScroll(value.id);
                setTimeout( function() {
                    Engine.initScrolls(value.id);
                }, 50);
            });
        }   

        Engine.updateDeprecatedUScroll();
        
        if (Utils.isNull(deuxiemePopup) || !deuxiemePopup || deuxiemePopup == "false") {
            $("#mainContent>div").bind('touchmove', function(e){e.preventDefault();});
        }
    },
    alert: function(message, height, popup2, timeout) {
        var height = Utils.isNotNull(height) ? height : "215px";
        $("#alert").css("height", height);
        Engine.addView("#alert","index.html:alert", { data : {texte: message, hideOk: Utils.isNotNull(timeout)} });
        Engine.popup("#alert", popup2);
        Engine.confirmCallback = null;
        Engine.confirmCallbackAnnuler = null;

        if (Utils.isNotNull(timeout)) {
            setTimeout(function(){Engine.fermerPopup(popup2);},timeout);
        }
    },
    confirmCallback: null,
    confirmCallbackAnnuler: null,
    confirm: function(message, callback, deuxiemePopup, height, fermerPopup, callbackAnnuler, libellesBoutons) {
        var deuxiemePopup = Utils.isNotNull(deuxiemePopup) ? deuxiemePopup : false;
        var fermerPopup = Utils.isNotNull(fermerPopup) ? fermerPopup : true;
        var height = Utils.isNotNull(height) ? height : "250px";
        var libellesBoutons = Utils.isNotNull(libellesBoutons) ? libellesBoutons : { confirmOK: Lang.confirmOK, confirmKO: Lang.confirmKO };
        $("#alert").css("height", height);
        $("#alert").css("padding", "10px");
        Engine.addView("#alert","index.html:confirm", { data : {texte: message, deuxiemePopup : deuxiemePopup }, libellesBoutons : libellesBoutons });
        Engine.popup("#alert", deuxiemePopup);
        $("#confirmOk").click(function(e) {
            e.stopPropagation();
            e.preventDefault();
            callback();
            if (fermerPopup) {
                Engine.fermerPopup(deuxiemePopup);
            }
        });
        $("#confirmKo").click(function(e) {
            e.stopPropagation();
            e.preventDefault();
            if (typeof callbackAnnuler === 'function') {
                callbackAnnuler();
            }
            Engine.fermerPopup(deuxiemePopup);
        });
    },
    confirmConflit: function(message, callback, callbackAnnuler) {
        Engine.addView("#conflitPopUp","index.html:conflit", { data : {texte: message} });
        Engine.popup("#conflitPopUp", true);
        $("#confirmConflitOk").click(function(e) {
            e.stopPropagation();
            e.preventDefault();
            callback();
        });
        $("#confirmConflitKo").click(function(e) {
            e.stopPropagation();
            e.preventDefault();
            callbackAnnuler();
        });
    },

    message: function(message, deuxiemePopup) {
        Engine.addView("#messagePopup","index.html:messagePopup", { data : {texte: message} });
        Engine.popup("#messagePopup", deuxiemePopup || false);
    },

    cleE: "",
    cleB: "",
    getJsonCle: function(callback, callbackError) {
        var data = {
            "name": (Utils.isNotNull(Engine.device.name)) ? Engine.device.name : "",
            "cordova": (Utils.isNotNull(Engine.device.cordova)) ? Engine.device.cordova : "",
            "platform": (Utils.isNotNull(Engine.device.platform)) ? Engine.device.platform : "",
            "uuid": (Utils.isNotNull(Engine.device.uuid)) ? Engine.device.uuid.toLowerCase() : "0",
            "version": (Utils.isNotNull(Engine.device.version)) ? Engine.device.version : ""
        };
        Utils.log("Data envoyé au WS (ci-dessous) ");
        Utils.log(data);
        Engine.showLoader();
        Utils.ajax("POST", Project.url, {
            'p_json': JSON.stringify({
                "date" : Utils.getDate(),
                "demande": "D001",
                "data": Utils.encryptServiceData(data, Project.cleI)
            }),
        }, function(json) {
            Engine.hideLoader();
            var json = Utils.decryptServiceData(json.data, Project.cleI);
            Engine.cleE = json.cle_e;
            Engine.cleB = json.cle_b;
            callback();
        }, function (json) {
            Engine.hideLoader();
            Engine.afficherErreurSI(json, callbackError);
        }, true);
    },
    login: "",
    mdp: "",
    getDemande: function(demande, data, callback, callbackError, bouchon, isAfficherConnectionError) {
        bouchon = (Utils.isNotNull(bouchon) && true === bouchon) ? true : false;
        if(bouchon) {
            Utils.jsonAjax(demande, function(json) {
//                Engine.fermerPopup();
                callback(json.data);
            });
        } else {
            if(!("login" in data) && !("mdp" in data)) {
                data["login"] = Engine.login;
                data["mdp"] = Engine.mdp;
            }
            if (!Utils.isNotBlank(data["mode_web"])) {
                data["mode_web"] = !Engine.dbSiteWeb && !Engine.isCordova ? "O" : "N";
            }
            Utils.log("Data envoyé au WS (ci-dessous) ");
            Utils.log(data);
            Utils.log('dfed');
            Utils.log(demande + " ====> " + JSON.stringify(data));
            var sJson = JSON.stringify({
                "date" : Utils.getDate(),
                "demande": demande,
                "uuid": Utils.encryptSha1(Engine.device.uuid),
                "data": Utils.encryptServiceData(data, Engine.cleE)
            });
            var regex = new RegExp(".{1," + Engine.tailleDecoupageFlux + "}", "g"); 
            var aJson = sJson.match(regex);
            jQuery.ajaxSettings.traditional = true;
            Utils.ajax("POST", Project.url, {
                'p_json': aJson
            }, function(retour) {
                var json = retour.data;
                try {
                    json = Utils.decryptServiceData(retour.data, Project.cleE);
                } catch(e){
                    console.error(e);
                }
                Utils.log(json);
                // Afficher une alerte si un code erreur est passé dans la réponse OK (ne doit pas bloquer le traitement)
                // Attention, dans le cas d'une demande D004, il ne faut pas effectuer de recherche dans le referentiel
                // pour éviter une boucle récursive.
                if (demande !== "D004" && Utils.isNotNull(retour.code_erreur)) {
                    Engine.findWebServiceErrorMessage(retour.code_erreur, function (msg) {
                        if (Utils.isNotBlank(msg)) {
                            // Affichage du message au second plan pour ne pas écraser celui de la synchro
                            Engine.alert(msg, null, true, 10000);
                        }
                        callback && callback(json);
                    });
                } else {
                    callback && callback(json);
                }
            }, function (json) {
                if (demande !== "D004") {
                    Engine.afficherErreurSI(json, callbackError);
                }
            }, isAfficherConnectionError);
        }
    },
    /**
     * Rechercher un message d'erreur dans le référentiel par son code réponse renvoyé par le SI
     */
    findWebServiceErrorMessage: function (code_erreur, callback) {
        Engine.getReferentiel("ref_typ", "FMI_COD_ERR", function(fmi_cod_err){
            var msg = null;
            for (var i = 0; i < fmi_cod_err.length; i++) {
                if (fmi_cod_err[i].ref_cod == code_erreur) {
                    msg = fmi_cod_err[i].ref_libelle;
                    break;
                }
            }
            callback && callback(msg);                        
        });
    },
    /**
     * Affichage de l'erreur retournée par le SI 
     */
    afficherErreurSI: function (json, callback) {
        var fn = function (fmi_cod_err) {
            var erreurMsg = "";
            // Recherche d'un message d'erreur dans le référentiel
            for (var i = 0; i < fmi_cod_err.length; i++) {
                if (Utils.parseInt(fmi_cod_err[i].ref_cod) === json.code_erreur) {
                    erreurMsg = fmi_cod_err[i].ref_libelle;
                }
            }
            // Message par défault si aucun message d'errreur n'a été trouvé dans le référentiel
            if (Utils.isBlank(erreurMsg)) {
                if (json.code_erreur === 3) {
                    erreurMsg = "Utilisateur et/ou Mot de passe invalide et/ou Mot de passe expiré";
                }
            }
            // TODO : passer le code et le message d'erreur dans le callback pour ne pas forcer l'affichage d'une alerte
            if (Utils.isNotBlank(erreurMsg)) {
                Engine.alert(erreurMsg);
            } else if (typeof callback === 'function') {
                callback(json);
            } else {
                Engine.alert("Erreur retournée par le SI de la FFF");
            }
        };
        if (!Engine.dbActif && json.code_erreur == 3) {            
            fn([]);
        } else {
            Engine.getReferentiel("ref_typ", "FMI_COD_ERR", function(fmi_cod_err){
                fn(fmi_cod_err);
            });
        }
    },
    selectCreate: function(dom, options) {
        // crétion des select via le plugin select2 si on est ni sur une tablette Windows ni sous IE
        if (!Engine.isWindows && !Engine.isCordova && !Utils.boolIsIe()) {
            dom.select2(options);
        }
    },
    selectSetVal: function(dom, data) {
        dom.val(data);
    },
    selectGetVal: function(dom) {
        if (!Engine.isWindows && !Engine.isCordova && !Utils.boolIsIe()) {
            if (dom.val() != undefined) {
                return dom.val();
            }
            return dom.select2("val");
        } else {
            return dom.val();
        }
    },
    scrollTop : function(idScroll) {
        if (Engine.isCordovaMobile()) {
            Engine.scrolls[idScroll].scrollTop();
        }
    },
    bindSignature: function(id, width, height) {
        try {
            $(id).jSignature('reset');
        } catch(ex) {
            Utils.log("Bind Signature non existante");
            $(id).jSignature({
                autoInit : true,
                format : "image/png",
                pen : "#000",
                penWidth : 2,
                border : "#AAA",
                width : (width != undefined) ? width : 400,
                height : (height != undefined) ? height : 400
            });
        }
        
        // Tweak the canvas opacity, causing it to redraw
        $(id + ' canvas').css('opacity', '0.99');
        var width = $(id + ' canvas').attr("width");
        $(id + ' canvas').css('opacity', '0.99');
        $(id + ' canvas').css("width", 1);
        $(id + ' canvas').hide();
    
        // Set the canvas opacity back to normal after 5ms
        setTimeout(function() {
            $(id + ' canvas').css('opacity', '1');
            $(id + ' canvas').css("width", width);
            $(id + ' canvas').show();
        }, 5);
        
        // no overflow hidden to the parents of the canvas
       	//$(id + ' canvas').parents("*").css("overflow", "visible");
        // set the canvas in position absolute
        //$(id + ' canvas').css("position", "absolute");
        //$(id + ' canvas').css("z-index", "1000000");
    },
    getSignature: function(id) {
        var img = $(id).jSignature('getData');
        $(id).jSignature('reset');
        return img;
    },
    getReferentiel: function(param, ref_typ, callback) {
        if (param === "ref_typ") {
            //si le param existe dans le Dto on le retourne
            if (ReferentielDto.fmi_ref.hasOwnProperty(ref_typ) && Utils.isNotNull(ReferentielDto.fmi_ref[ref_typ]) && ReferentielDto.fmi_ref[ref_typ].length > 0) {
                callback(ReferentielDto.fmi_ref[ref_typ]);
            } else {
                Engine.getRefFromBase("ref_typ", ref_typ, true, callback);
            }
        } else {
            Engine.getRefFromBase(param, ref_typ, false, callback);
        }
    },
    getRefFromBase : function(param, ref_typ, addToDto, callback) {
        DataBase.findBy("FMI_REF", param, ref_typ, function(json){
            cmp = function(x, y) {
                return x > y ? 1 : x < y ? -1 : 0; 
            };
            var toSort = Engine.dbActif ? json : json.param;
            toSort.sort(function(a, b) {
                return cmp(
                    [cmp(a.ref_typ, b.ref_typ), cmp(parseInt(a.ref_sort, 10), parseInt(b.ref_sort, 10))], 
                    [cmp(b.ref_typ, a.ref_typ), cmp(parseInt(b.ref_sort, 10), parseInt(a.ref_sort, 10))]
                );
            });
            if (addToDto) {
                ReferentielDto.fmi_ref[ref_typ] = toSort;
            }
            if (typeof callback === 'function') {
                callback(toSort);
            }
        });
    }

};

