var EquipeLicenceService = {
    licenceMatchQualificationOk: function (li_dat_qualif) {
        var ma_dat = EquipeDto.data.ma_dat || MatchDto.match.ma_dat;
        if (Utils.isNotBlank(li_dat_qualif)) {
            return Utils.stringToDate(ma_dat)-Utils.stringToDate(li_dat_qualif) >= 0;
        } else {
            return true;
        }
    },
    /**
     * Retourne le niveau d'erreur d'une licence d'une licences
     * 
     * @param {Object} licence  Les informations de licence
     * @return {Integer} Le code erreur de la licence
     *      0 : Pas d'erreur
     *      1 : La licence
     *      2 : La licence à une information de mutation ou n'est pas active ou n'est pas qualifiée pour le match
     *      3 : La licence comporte des sanctions
     */
    getNiveauErreurLicence: function (licence) {
        if (EquipeDragAndDropService.joueurSanctione(licence.in_no)) {
            return 3;
        } else if (licence.li_statut === "N" || !EquipeLicenceService.licenceMatchQualificationOk(licence.li_dat_qualif)) {
            return 2;
        } else if (Utils.isNotBlank(licence.li_mut) || Utils.isNotBlank(licence.ic_cachet_lib)) {
            return 1;
        } else {
            return 0;
        }
    },
    isIndividuExiste : function(inNo) {
        return $("#listeJoueur li[data-inno='"+inNo+"']").length !== 0;
    },
    /**
     * Retourne le li_no d'un individu pour les types de licences données (J/F/D)
     * @param {String} inNo
     * @param {Array} typeLicenceList
     * @return {String} un li_no ou une chaîne vide si aucune n'a été trouvé
     */
    getLiNoByInNoAndLicenceType: function(inNo, typeLicenceList) {
        return $("#listeJoueur li[data-inno='"+inNo+"']")
            .filter(function () { return Utils.inArray($(this).find(".tl_uti_typ_lic").val(), typeLicenceList); })
            .attr("id") || "";
    },
    isStaff: function(idLicence) {
        var fonction = $("#" + idLicence).find(".infosJoueurHidden .tl_uti_typ_lic").val();
        return (Utils.isNotBlank(fonction) && !(fonction === "J" || fonction === "F"));
    },
    showLicences : function(licences, callback, sourceController) {
        var sourceController = Utils.isNull(sourceController) ? ".EquipeController" : sourceController;
        
        Engine.addView(sourceController + " #scrollableContainer", "equipe.html:afficherLicences");
        
        for (var i=0; i<licences.length; i++) {
            //si c'est la préparation de l'équipe lorsqu'on vient de la feuille de match,
            //on ne doit pas afficher les couleurs des boutons infos
            var preparationSolo = Engine.filAriane.length > 1 && Engine.filAriane[Engine.filAriane.length-2] != "feuilleDeMatch.html";
            var codeErreurLicence = 0;
            if (preparationSolo) {
                codeErreurLicence = EquipeLicenceService.getNiveauErreurLicence(licences[i]);
            }
            var infoLicenceClassName = "";
            if (codeErreurLicence === 1) {
                infoLicenceClassName = "infoLicenceJaune";
            } else if (codeErreurLicence === 2) {
                infoLicenceClassName = "infoLicenceOrange";
            } else if (codeErreurLicence === 3) {
                infoLicenceClassName = "infoLicenceRouge";
            }
            if (Utils.isNotNull(licences[i].li_no) && Utils.isNotBlank(licences[i].li_no) && licences[i].li_no !== "null") {
                Engine.addView(sourceController + " #scrollableContainer #listeJoueur", "equipe.html:licenceItem", {
                    joueur : licences[i],
                    infoLicenceClassName : infoLicenceClassName,
                    hideJoueur : false
                }, false, true);
            } else {
                Engine.addView(sourceController + " #scrollableContainer #listeJoueur", "equipe.html:licenceItem", {
                    joueur : licences[i],
                    infoLicenceClassName : infoLicenceClassName,
                    hideJoueur : true
                }, false, true);
            }
        }       
        
        //************************************************//
        //************* CREATION DES FILTRES *************//
        //************************************************//
        var fonctions = [];
        $("#effectif li").each(function() {
            var fonction = $.trim($(this).find(".sc_lib").val());
            if (!Utils.inArray(Utils.capitalize(fonction), fonctions)) {
                fonctions.push(Utils.capitalize(fonction));
            }
        });
        var html = "<option value='0'>" + Lang.equipe.fonction + "</option>";
        fonctions.sort();
        for (var i in fonctions) {
            html += "<option value='" + fonctions[i] + "'>" + fonctions[i] + "</option>";
        }
        $("#selectFonction").html(html);
        Engine.selectSetVal($("#selectFonction"), 0);

        var htmlGroupe = "<option value='0'>" + Lang.equipe.groupe + "</option>";
        htmlGroupe += "<option value='filtreGroupePlace'>" + Lang.equipe.groupePlace + "</option>";

        if(!Engine.dbActif) {
            DataBase.findBy("CLUB_GROUPE", "cl_no", EquipeDto.utilisateur.cl_no, function(json) {
                for (i in json.cl_grp) {
                    htmlGroupe += "<option value='filtreGroupeN" + json.cl_grp[i].cgr_no + "'>" + json.cl_grp[i].cgr_nom + "</option>";
                }
                for (i in json.cl_grp_in) {
                    $("#listeJoueur").find("[data-inno=" + json.cl_grp_in[i].in_no + "]").each(function() {
                        $(this).find(".infosJoueurHidden").append("<span class='filtreGroupeN" + json.cl_grp_in[i].cgr_no + "'></span>");
                    });
                }
                $("#selectGroupe").html(htmlGroupe);
                Engine.selectSetVal($("#selectGroupe"), 0);
            });
        } else {
            DataBase.findBy("CLUB_GROUPE", "cl_no", EquipeDto.utilisateur.cl_no, function(json) {
                for (i in json) {
                    htmlGroupe += "<option value='filtreGroupeN" + json[i].cgr_no + "'>" + json[i].cgr_nom + "</option>";
                    DataBase.findBy("CLUB_GROUPE_INDIVIDU", "cgr_no", json[i].cgr_no, function(cl_grp_in) {
                        for (j in cl_grp_in) {
                            $("#listeJoueur").find("[data-inno=" + cl_grp_in[j].in_no + "]").each(function() {
                                $(this).find(".infosJoueurHidden").append("<span class='filtreGroupeN" + cl_grp_in[j].cgr_no + "'></span>");
                            });
                        }
                    });
                }
                $("#selectGroupe").html(htmlGroupe);
                Engine.selectSetVal($("#selectGroupe"), 0);
            });
        }
        callback();
    },
    refreshEquipes : function(numeroLicences, numeroMatch, callback) {
        Engine.message("Chargement des licences");
        var timeCheck = setTimeout(function(){
            Engine.fermerPopup();
            Engine.alert("Une erreur est survenue pendant le chargement des licences. Veuillez retourner sur l'accueil et recommencer.\nSi le problème persiste, re-synchronisez votre application", "270px");
        }, 35000);
        DataBase.findLicence(numeroMatch, EquipeDto.utilisateur.cl_no, numeroLicences.split(";"), function(dataReturn) {
            cmp = function(x, y) {
                return x > y ? -1 : x < y ? 1 : 0; 
            };
            dataReturn.sort(function(a, b) {
                return cmp( 
                    [cmp(a.in_nom, b.in_nom), cmp(a.in_prenom, b.in_prenom)], 
                    [cmp(b.in_nom, a.in_nom), cmp(b.in_prenom, a.in_prenom)]
                );
            });
            EquipeLicenceService.showLicences(dataReturn, function() { 
                EquipeCompositionService.chargerComposition(EquipeDto.data.ma_no, EquipeDto.data.cfm_no, EquipeDto.utilisateur.cl_no, function () {
                    clearTimeout(timeCheck);
                    Engine.fermerPopup();
                });
            });
        });
    },
    getLicenceFromBase : function(liNoRestant) {
    },
    getSanctions : function(numeroMatch) {
         DataBase.findBy("SANCTION", "ma_no", numeroMatch, function(json) {
                SanctionDto.listSanctions = json;
         });
    },
    afficherLicencePopup: function(numLicence){
        var elementLi = $("#" + numLicence);
        Engine.getReferentiel("ref_typ","FMI_LI_ETAT", function(){
            var fmi_li_etat = "";
            for (i in ReferentielDto.fmi_ref.FMI_LI_ETAT) {
                if (ReferentielDto.fmi_ref.FMI_LI_ETAT[i].ref_cod == elementLi.find(".infosJoueurHidden .li_etat").val()) {
                    fmi_li_etat = ReferentielDto.fmi_ref.FMI_LI_ETAT[i].ref_libelle;
                }
                if (ReferentielDto.fmi_ref.FMI_LI_ETAT[i].ref_cod == elementLi.find(".infosJoueurHidden .li_etat").text()) {
                    fmi_li_etat = ReferentielDto.fmi_ref.FMI_LI_ETAT[i].ref_libelle;
                }
            }
            var data = { 
                li_no : elementLi.find(".infosJoueurHidden .li_no").val(),
                in_no : elementLi.find(".infosJoueurHidden .in_no").val(),
                in_nom : elementLi.find(".infosJoueurHidden .in_nom").val(),
                in_prenom: elementLi.find(".infosJoueurHidden .in_prenom").val(),
                sc_lib : elementLi.find(".infosJoueurHidden .sc_lib").val(),
                li_di_fonc : elementLi.find(".infosJoueurHidden .li_di_fonc").val(),
                in_dat_nais : elementLi.find(".infosJoueurHidden .in_dat_nais").val(),
                ic_cachet_lib : elementLi.find(".infosJoueurHidden .ic_cachet_lib").val(),
                in_photo : Utils.prepareBase64Image(elementLi.find(".infosJoueurHidden .in_photo").val(), 'img/person.png'),
                sa_lib : elementLi.find(".infosJoueurHidden .sa_lib").val(),
                cl_nom : elementLi.find(".infosJoueurHidden .cl_nom").val(),
                li_dat_enr : elementLi.find(".infosJoueurHidden .li_dat_enr").val(),
                li_dat_qualif : elementLi.find(".infosJoueurHidden .li_dat_qualif").val(),
                in_nat : elementLi.find(".infosJoueurHidden .in_nat").val(),
                cg_nom : elementLi.find(".infosJoueurHidden .cg_nom").val(),
                li_certif_medic_abs : elementLi.find(".infosJoueurHidden .li_certif_medic_abs").val(),
                li_etat : elementLi.find(".infosJoueurHidden .li_etat").val(),
                cq_cl_info : elementLi.find(".infosJoueurHidden .cq_cl_info").val(),
                li_natu_deman : elementLi.find(".infosJoueurHidden .li_natu_deman").val(),
                cl_cod : elementLi.find(".infosJoueurHidden .cl_cod").val(),
                li_certif_medic : elementLi.find(".infosJoueurHidden .li_certif_medic").val(),
                li_statut : elementLi.find(".infosJoueurHidden .li_statut").val(),
                fmi_li_etat : fmi_li_etat,
                li_cl_grpt : elementLi.find(".infosJoueurHidden .li_cl_grpt").val()
            };
            
            if (data.ic_cachet_lib) {
                data.ic_cachet_lib.replace("@@@" , "<br />");
            }
            
            //on parcours le DTO à la recherche d'une sanction pour le joueur
            var dataSanctions = [];
            if(SanctionDto.fromPreparer == false){
                for (i in SanctionDto.listSanctions) {
                    if(SanctionDto.listSanctions[i].in_no == elementLi.find(".infosJoueurHidden .in_no").val()){
                        dataSanctions.push(SanctionDto.listSanctions[i]);
                    }    
                }
                data["list_sanctions"] = dataSanctions;
            }
            var widthDiv = "950px";
            if(dataSanctions.length == 0) {
                var widthDiv = "600px";
            }
            $("#ficheJoueur").css("width", widthDiv);
            
            Engine.addView("#ficheJoueur", "equipe.html:afficherLicence", { infoJoueur : data }, true);
            Engine.popup("#ficheJoueur");
        });
    },
    controleNumMaillot : function(erreur, numJoueur) {
        if (isNaN(numJoueur) || !Utils.isIntegerPositif(numJoueur)) {
            erreur.push(Lang.equipe.maillotFormat);
        } else {
            var maillotOk = true;
            $(".emplacementLibre").each(function() {
                var numeroMaillot = $(this).find(".numJoueur").text();
                if (Utils.isNotBlank(numeroMaillot) && $(this).attr("id") != EquipeDto.composition.joueurCompositionSelection && numeroMaillot == numJoueur) {
                    maillotOk = false;
                }
            });
            if (!maillotOk) {
                erreur.push(Lang.equipe.maillotUtilise);
            } else {
                if ($("#" + EquipeDto.composition.joueurCompositionSelection).find(".numJoueur").length == 0) {
                    $("#" + EquipeDto.composition.joueurCompositionSelection).children().append("<div class='numJoueur'>" + numJoueur + "</div>");
                } else {
                    $("#" + EquipeDto.composition.joueurCompositionSelection).find(".numJoueur").text(numJoueur);
                }
            }
        }
    },
    /**
     * Rechercher des licences.
     * En mode tablette, la recherche s'effectue en base.
     * En mode web elle s'effectue sur la liste de gauche c-a-d parmis les joueurs inscris pour le match.
     * 
     * @param {Object} in_no Le in_no du joueur à rechercher
     * @param {Object} callback
     */
    rechercherLicences: function (in_no, callback) {
        if (Engine.dbActif) {
            DataBase.findBy("CLUB_LICENCE", "in_no", in_no, function (licences) { 
                var licence = null;
                if (licences.length === 1) {
                    if($("#"+ licences[0].li_no).length == 0) {
                        licences[0].li_no = "null";
                    } 
                    licence = licences[0];
                } else if (licences.length > 1) {
                    // Si il y a plus d'une licence pour ce in_no, il faut choisir celle qui est encadrant
                    for (var i = 0; i<licences.length; i++) {
                        if (licences[i].tl_uti_typ_lic !== "F" && licences[i].tl_uti_typ_lic !== "J") {
                            if($("#"+ licences[i].li_no).length == 0) {
                                licences[i].li_no = "null";
                            } 
                            licence = licences[i];
                        }
                    }
                }
                if (typeof callback === 'function') {
                    callback(licence);
                }
            });
        } else {
            var liLicences = $("#listeJoueur li[data-inno='"+in_no+"']");
            var liLicence = null;
            if (liLicences.length === 1) {
                liLicence = liLicences.first();
            } else if (liLicences.length > 1) {
                // Si il y a plus d'une licence pour ce in_no, il faut choisir celle qui est encadrant
                liLicence = $("#listeJoueur li[data-inno='"+in_no+"']").filter(function () {
                    var typ_lic = $(this).find('.tl_uti_typ_lic').val();
                    return type_lic !== "F" && type_lic !== "J";
                }).first();
            }
            if (Utils.isNotNull(liLicence)) {
                var data = {
                    li_no : liLicence.find('.li_no').val(),
                    in_no : liLicence.find('.in_no').val(),
                    in_nom : liLicence.find('.in_nom').val(),
                    in_prenom : liLicence.find('.in_prenom').val(),
                    in_photo : liLicence.find('.in_photo').val(),
                    in_dat_nais : liLicence.find(".in_dat_nais").val()
                };
                if (typeof callback === 'function') {
                    callback(data);
                }
            } else {
                if (typeof callback === 'function') {
                    callback(null);
                }
            }
        }
    }
};
